package com.zhongshu.reward.server.core.service;

import com.github.microservice.auth.client.content.ResultState;
import com.github.microservice.productcenter.client.model.ProductRuleModel;
import com.github.microservice.productcenter.client.model.RebateModel;
import com.github.microservice.productcenter.client.ret.ResultContent;
import com.github.microservice.productcenter.client.service.ProductDataService;
import com.github.microservice.productcenter.client.service.ProductRuleService;
import com.github.microservice.productcenter.client.type.RebateType;
import com.zhongshu.payment.client.model.order.v2.GoodsResultModel;
import com.zhongshu.payment.client.service.v2.OrderService2;
import com.zhongshu.reward.client.type.ReceiptsStatus;
import com.zhongshu.reward.client.type.ReceiptsType;
import com.zhongshu.reward.server.core.dao.WalletDao;
import com.zhongshu.reward.server.core.dao.WalletReceiptsDao;
import com.zhongshu.reward.server.core.domain.Wallet;
import com.zhongshu.reward.server.core.domain.WalletReceipts;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;

/**
 * @author wjf
 * @date 2024/8/23
 */
@Service
@Slf4j
public class GoodsReceiptsService {

    @Autowired
    WalletDao walletDao;

    @Autowired
    WalletService walletService;

    @Autowired
    WalletReceiptsDao walletReceiptsDao;

    @Autowired
    ProductDataService productDataService;

    @Autowired
    OrderService2 orderService2;


    /**
     * 购物返利 todo// 1、获取产品库 返利规则 2、获取订单详情、获取商品详情
     */
    @Transactional
    public void receipts(String userId, String orderNo, String payGoodsId, Long operationTime){

        com.github.microservice.auth.client.content.ResultContent<GoodsResultModel> goodsResultModelResultContent = orderService2.qrGoods(payGoodsId);
        if (!ResultState.Success.equals(goodsResultModelResultContent.getState())){
            log.info("查询商品信息失败：id:{}", payGoodsId);
            return;
        }
        GoodsResultModel goodsResultModel = goodsResultModelResultContent.getContent();
        if (StringUtils.isEmpty(goodsResultModel.getGoodsId())){
            log.info("商品无对应产品库id:{}", payGoodsId);
            return;
        }

        BigDecimal total = getReceiptsTotal(goodsResultModel.getPayAmount(), goodsResultModel.getGoodsId(), goodsResultModel.getJobFlowMap().name());
        if (BigDecimal.ZERO.compareTo(total) == 0){
            log.info("商品返利金额为0, id:{}, productId:{}", payGoodsId, goodsResultModel.getGoodsId());
            return;
        }

        Wallet wallet = walletService.getWalletByUserId(userId);

        WalletReceipts walletReceipts = new WalletReceipts();
        walletReceipts.setWallet(wallet);
        walletReceipts.setReceiptsType(ReceiptsType.REBATE);
        walletReceipts.setEstimatedTime(operationTime + 2*24*60*60*1000);
        walletReceipts.setTotal(total);
        walletReceipts.setOutTradeNo(orderNo);
        walletReceipts.setGoodsId(goodsResultModel.getGoodsId());
        walletReceipts.setPaymentGoodsId(payGoodsId);
        walletReceipts.setStatus(ReceiptsStatus.WAIT);

        walletReceiptsDao.save(walletReceipts);
        walletDao.updateIncWaitAmount(wallet.getId(), total);
        log.info("购物返利入账");
    }

    private BigDecimal getReceiptsTotal(BigDecimal goodsTotal, String goodsId, String channel) {
        BigDecimal total = BigDecimal.ZERO;
        ResultContent<RebateModel> productRulerResult = productDataService.rebate(goodsId, channel);
        if (productRulerResult.getContent()==null){
            return total;
        }

        RebateType rebate = productRulerResult.getContent().getRebate();
        if (rebate!=null && rebate.equals(RebateType.Fixed)){
            total = productRulerResult.getContent().getValue();
        }else if (rebate!=null && rebate.equals(RebateType.Ratio)){
            total = goodsTotal.multiply(productRulerResult.getContent().getValue()).setScale(2, RoundingMode.HALF_UP);
        }
        return total;
    }

//    @Nullable
//    private ProductRuleModel getRule(String goodsId, String channel) {
//        ResultContent<RebateModel> productRulerResult = productDataService.rebate(goodsId, channel);
//
//    }


    /**
     * 取消购物返利
     * @param orderNo 订单号
     */
    @Transactional
    public void cancel(String orderNo, String payGoodsId, Long operateTime){
        WalletReceipts walletReceipts = walletReceiptsDao.findByOutTradeNoAndPaymentGoodsIdAndStatus(orderNo, payGoodsId, ReceiptsStatus.WAIT);
        if (ObjectUtils.isEmpty(walletReceipts)){
            log.info("无购物返利订单：{}", payGoodsId);
            return;
        }
        if (operateTime > walletReceipts.getEstimatedTime()){
            return;
        }
        walletReceipts.setStatus(ReceiptsStatus.CANCEL);
        walletReceiptsDao.save(walletReceipts);
        walletDao.updateIncWaitAmount(walletReceipts.getWallet().getId(), walletReceipts.getTotal().negate());
    }

    /**
     * 入账结算
     */
    public void settle(){
        //todo 查询到期入账的订单
        List<WalletReceipts> waitSettle = walletReceiptsDao.findByWaitSettle(ReceiptsStatus.WAIT, ReceiptsType.REBATE, new Date().getTime());
        if (waitSettle == null || waitSettle.isEmpty()){
            return;
        }
        waitSettle.forEach(walletReceipts->{
            walletReceipts.setReceiptsTime(new Date().getTime());
            walletReceipts.setStatus(ReceiptsStatus.RECEIPTS);
            walletReceiptsDao.save(walletReceipts);
            walletDao.updateIncWaitAmount(walletReceipts.getWallet().getId(), walletReceipts.getTotal().negate());
            walletDao.updateIncAmount(walletReceipts.getWallet().getId(), walletReceipts.getTotal());
            log.info("结算商品返利：{}", walletReceipts.getId());
        });
    }
}
